@@ -9,19 +9,26 @@ import qualified Data.MyText as T
 import Data.MyText (Text)
 import Control.Monad
 import Control.Monad.Instances
+import Control.Monad.Trans.Reader
+import Control.Monad.Trans.Class
+import Data.Functor.Identity
 
-import Text.ParserCombinators.Parsec hiding (Parser)
-import Text.ParserCombinators.Parsec.Token
-import Text.ParserCombinators.Parsec.Language
-import Text.ParserCombinators.Parsec.ExprFail
+
+import Text.Parsec
+import Text.Parsec.Char
+import Text.Parsec.Token
+import Text.Parsec.Combinator
+import Text.Parsec.Language
+import Text.Parsec.ExprFail
 import System.Exit
 import Control.Applicative ((<*>),(<$>))
+import Control.DeepSeq
 import Data.List
 import Data.Maybe
 import Data.Char
 import Data.Time.Clock
 import Data.Time.LocalTime
-import Data.Time.Calendar (toGregorian)
+import Data.Time.Calendar (toGregorian, fromGregorian)
 import Data.Time.Calendar.WeekDate (toWeekDate)
 import Data.Time.Format (formatTime)
 import System.Locale (defaultTimeLocale, iso8601DateFormat)
@@ -32,12 +39,12 @@ import Text.Printf
 type Categorizer = TimeLog CaptureData -> TimeLog (Ctx, ActivityData)
 type Rule = Ctx -> ActivityData
 
-type Parser a = CharParser () a
+type Parser = ParsecT String () (ReaderT TimeZone Identity)
+
 
 data Ctx = Ctx
         { cNow :: TimeLogEntry CaptureData
-        , cPast :: [TimeLogEntry CaptureData]
-        , cFuture :: [TimeLogEntry CaptureData]
+        , cCurrentWindow :: Maybe (Bool, Text, Text)
         , cWindowInScope :: Maybe (Bool, Text, Text)
         , cSubsts :: [Text]
         , cCurrentTime :: UTCTime
@@ -45,6 +52,9 @@ data Ctx = Ctx
         }
   deriving (Show)
 
+instance NFData Ctx where
+    rnf (Ctx a b c d e f) = a `deepseq` b `deepseq` c `deepseq` e `deepseq` e `deepseq` f `deepseq` ()
+
 type Cond = CtxFun [Text]
 
 type CtxFun a = Ctx -> Maybe a
@@ -56,6 +66,8 @@ data CondPrim
         | CondTime (CtxFun NominalDiffTime)
         | CondDate (CtxFun UTCTime)
         | CondCond (CtxFun [Text])
+        | CondStringList (CtxFun [Text])
+        | CondRegexList (CtxFun [RE.Regex])
 
 newtype Cmp = Cmp (forall a. Ord a => a -> a -> Bool)
 
@@ -64,27 +76,24 @@ readCategorizer filename = do
         content <- readFile filename
         time <- getCurrentTime
         tz <- getCurrentTimeZone
-        case parse (between (return ()) eof parseRules) filename content of
+        case flip runReader tz $
+            runParserT (between (return ()) eof parseRules) () filename content of
           Left err -> do
                 putStrLn "Parser error:"
                 print err
                 exitFailure
           Right cat -> return $
-                ((fmap . fmap) (mkSecond (postpare . cat)) . prepare time tz)
+                (map (fmap (mkSecond (postpare . cat))) . prepare time tz)
 
-applyCond :: String -> TimeLogEntry (Ctx, ActivityData) -> Bool
-applyCond s = 
-        case parse (do {c <- parseCond; eof ; return c}) "commad line parameter" s of
+applyCond :: String -> TimeZone -> TimeLogEntry (Ctx, ActivityData) -> Bool
+applyCond s tz = 
+        case flip runReader tz $ runParserT (do {c <- parseCond; eof ; return c}) () "commad line parameter" s of
           Left err -> error (show err)
-          Right c    -> isJust . c . fst . tlData
+          Right c  -> isJust . c . fst . tlData
 
 prepare :: UTCTime -> TimeZone -> TimeLog CaptureData -> TimeLog Ctx
-prepare time tz tl = go' [] tl tl
-  where go' past [] []
-                = []
-        go' past (this:future) (now:rest)
-                = now {tlData = Ctx now past future Nothing [] time tz } :
-                  go' (this:past) future rest
+prepare time tz = map go
+  where go now  = now {tlData = Ctx now (findActive (cWindows (tlData now))) Nothing [] time tz }
 
 -- | Here, we filter out tags appearing twice, and make sure that only one of
 --   each category survives
@@ -93,8 +102,20 @@ postpare = nubBy go
   where go (Activity (Just c1) _) (Activity (Just c2) _) = c1 == c2
         go a1                     a2                     = a1 == a2
 
-lang :: TokenParser ()
-lang = haskell
+lang :: GenTokenParser String () (ReaderT TimeZone Identity)
+lang = makeTokenParser $ LanguageDef
+                { commentStart   = "{-"
+                , commentEnd     = "-}"
+                , commentLine    = "--"
+                , nestedComments = True
+                , identStart     = letter
+                , identLetter	 = alphaNum <|> oneOf "_'"
+                , opStart	 = oneOf ":!#$%&*+./<=>?@\\^|-~"
+                , opLetter	 = oneOf ":!#$%&*+./<=>?@\\^|-~"
+                , reservedOpNames= []
+                , reservedNames  = []
+                , caseSensitive  = True
+                }
 
 parseRules :: Parser Rule
 parseRules = do 
@@ -155,7 +176,7 @@ parseCond = do cp <- parseCondExpr
                 cp         -> fail $ printf "Expression of type %s" (cpType cp)
 
 parseCondExpr :: Parser CondPrim
-parseCondExpr  = buildExpressionParser [
+parseCondExpr = buildExpressionParser [
                 [ Prefix (reservedOp lang "!" >> return checkNot) ],
                 [ Prefix (reserved lang "day of week" >> return evalDayOfWeek)
                 , Prefix (reserved lang "day of month" >> return evalDayOfMonth)
@@ -179,12 +200,18 @@ cpType (CondInteger _) = "Integer"
 cpType (CondTime _) = "Time"
 cpType (CondDate _) = "Date"
 cpType (CondCond _) = "Condition"
+cpType (CondStringList _) = "List of Strings"
+cpType (CondRegexList _) = "List of regular expressions"
 
 checkRegex :: CondPrim -> CondPrim -> Erring CondPrim
 checkRegex (CondString getStr) (CondRegex getRegex) = Right $ CondCond $ \ctx -> do
         str <- getStr ctx
         regex <- getRegex ctx
         tail <$> RE.match regex str [RE.exec_no_utf8_check]
+checkRegex (CondString getStr) (CondRegexList getRegexList) = Right $ CondCond $ \ctx -> do
+        str <- getStr ctx
+        regexes <- getRegexList ctx
+        tail <$> msum (map (\regex -> RE.match regex str [RE.exec_no_utf8_check]) regexes)
 checkRegex cp1 cp2 = Left $
         printf "Cannot apply =~ to an expression of type %s and type %s"
                (cpType cp1) (cpType cp2)
@@ -225,18 +252,28 @@ checkCmp (Cmp (?)) (CondTime getT1) (CondTime getT2) = Right $ CondCond $ \ctx -
         t2 <- getT2 ctx
         guard (t1 ? t2)
         return []
+checkCmp (Cmp (?)) (CondDate getT1) (CondDate getT2) = Right $ CondCond $ \ctx -> do
+        t1 <- getT1 ctx
+        t2 <- getT2 ctx
+        guard (t1 ? t2)
+        return []
 checkCmp (Cmp (?)) (CondString getS1) (CondString getS2) = Right $ CondCond $ \ctx -> do
         s1 <- getS1 ctx
         s2 <- getS2 ctx
         guard (s1 ? s2)
         return []
+checkCmp (Cmp (?)) (CondString getS1) (CondStringList getS2) = Right $ CondCond $ \ctx -> do
+        s1 <- getS1 ctx
+        sl <- getS2 ctx
+        guard (any (s1 ?) sl)
+        return []
 checkCmp _ cp1 cp2 = Left $
         printf "Cannot compare expressions of type %s and type %s"
                (cpType cp1) (cpType cp2)
 
 checkCurrentwindow :: CondPrim -> Erring CondPrim
 checkCurrentwindow (CondCond cond) = Right $ CondCond $ \ctx -> 
-        cond (ctx { cWindowInScope = findActive (cWindows (tlData (cNow ctx))) })
+        cond (ctx { cWindowInScope = cCurrentWindow ctx })
 checkCurrentwindow cp = Left $
         printf "Cannot apply current window to an expression of type %s"
                (cpType cp)
@@ -309,6 +346,14 @@ parseCmp = choice $ map (\(s,o) -> reservedOp lang s >> return o)
 parseCondPrim :: Parser CondPrim
 parseCondPrim = choice
         [ parens lang parseCondExpr
+        , brackets lang (choice [
+            (do list <- commaSep1 lang (stringLiteral lang)
+                return $ CondStringList (const (Just (map T.pack list)))
+            ) <?> "list of strings",
+            (do list <- commaSep1 lang parseRegex
+                return $ CondRegexList (const (Just list))
+            ) <?> "list of regular expressions"
+            ])
         , char '$' >> choice 
              [ do backref <- natural lang
                   return $ CondString (getBackref backref)
@@ -336,6 +381,8 @@ parseCondPrim = choice
              return $ CondString (const (Just str))
         , try $ do time <- parseTime <?> "time" -- backtrack here, it might have been a number
                    return $ CondTime (const (Just time))
+        , try $ do date <- parseDate <?> "date" -- backtrack here, it might have been a number
+                   return $ CondDate (const (Just date))
         , do num <- natural lang <?> "number"
              return $ CondInteger (const (Just num))
         ]
@@ -392,6 +439,19 @@ parseTime = fmap fromIntegral $ lexeme lang $ do
                let hour = maybe h ((10*h)+) mh
                return $ (hour * 60 + m1 * 10 + m2) * 60
 
+parseDate :: Parser UTCTime
+parseDate = lexeme lang $ do
+    tz <- lift ask
+    year <- read <$> count 4 digit
+    char '-'
+    month <- read <$> count 2 digit
+    char '-'
+    day <- read <$> count 2 digit
+    time <- option 0 parseTime
+    let date = LocalTime (fromGregorian year month day) (TimeOfDay 0 0 0)
+    return $ addUTCTime time $ localTimeToUTC tz date
+
+
 parseSetTag :: Parser Rule
 parseSetTag = lexeme lang $ do
                  firstPart <- parseTagPart 
@@ -454,7 +514,7 @@ getBackref n ctx = listToMaybe (drop (fromIntegral n-1) (cSubsts ctx))
 getVar :: String -> CtxFun Text
 getVar v ctx | "current" `isPrefixOf` v = do
                 let var = drop (length "current.") v
-                win <- findActive $ cWindows (tlData (cNow ctx))
+                win <- cCurrentWindow ctx
                 getVar var (ctx { cWindowInScope = Just win })
 getVar "title"   ctx = do
                 (_,t,_) <- cWindowInScope ctx
