@@ -48,6 +48,9 @@ data Activity = Activity
         }
   deriving (Ord, Eq)
 
+instance NFData Activity where
+    rnf (Activity a b) = a `deepseq` b `deepseq` ()
+
 -- | An activity with special meaning: ignored by default (i.e. for idle times)
 inactiveActivity = Activity Nothing "inactive"
 
@@ -91,7 +94,7 @@ instance Binary UTCTime where
  get = do
         d <- get
         t <- get
-        return $ UTCTime (ModifiedJulianDay d) (fromRational t)
+        return $ UTCTime (ModifiedJulianDay d) ({-# SCC diffTimeFromRational #-} fromRational t)
 
 instance ListOfStringable CaptureData where
   listOfStrings = concatMap (\(b,t,p) -> [t,p]) . cWindows
