@@ -15,6 +15,8 @@ import Data.Char
 import System.Directory
 import Control.Exception
 import Prelude hiding (catch)
+import Control.DeepSeq
+import System.Posix.Files
 
 import qualified Data.ByteString.Lazy as BS
 import Data.Maybe
@@ -28,6 +30,7 @@ runLogger filename delay action = flip fix Nothing $ \loop prev -> do
         entry <- action
         date <- getCurrentTime
         createTimeLog False filename
+        setFileMode filename (ownerReadMode `unionFileModes` ownerWriteMode)
         appendTimeLog filename prev (TimeLogEntry date delay entry)
         threadDelay (fromIntegral delay * 1000)
         loop (Just entry)
@@ -86,20 +89,25 @@ recoverTimeLog filename = do
                         )
           where strs = maybe [] listOfStrings prev
 
-readTimeLog :: ListOfStringable a => FilePath -> IO (TimeLog a)
+readTimeLog :: (NFData a, ListOfStringable a) => FilePath -> IO (TimeLog a)
 readTimeLog filename = do
         content <- BS.readFile filename
-        return $ runGet start content
-  where start = do
-                startString <- getLazyByteString (BS.length magic)
-                if startString == magic
-                 then go Nothing
-                 else error $
-                        "Timelog starts with unknown marker " ++
-                        show (map (chr.fromIntegral) (BS.unpack startString))
-        go prev = do v <- ls_get strs
-                     m <- isEmpty
-                     if m then return [v]
-                          else (v :) <$> go (Just (tlData v))
-          where strs = maybe [] listOfStrings prev
+        return $ parseTimeLog content
+
+parseTimeLog :: (NFData a, ListOfStringable a) => BS.ByteString -> TimeLog a
+parseTimeLog input = 
+    if startString == magic
+       then go Nothing rest off
+       else error $
+            "Timelog starts with unknown marker " ++
+            show (map (chr.fromIntegral) (BS.unpack startString))
+  where                
+    (startString, rest, off) = runGetState (getLazyByteString (BS.length magic)) input 0
+    go prev input off =
+        let (v, rest, off') = runGetState (ls_get strs) input off
+        in v `deepseq`
+           if (BS.null rest)
+           then [v]
+           else v : go (Just (tlData v)) rest off'
+      where strs = maybe [] listOfStrings prev
 
