@@ -11,11 +11,18 @@ import Data.Char (toLower)
 import Text.Printf
 import Data.Version (showVersion)
 import Control.DeepSeq
+import Control.Applicative
+import qualified Data.ByteString.Lazy as BS
+import Data.ByteString.Lazy.Progress
+import System.Posix.Files
+import System.ProgressBar
+import TermSize
 
 import TimeLog
 import Categorize
 import Stats
 import CommonStartup
+import LeftFold
 
 import Paths_arbtt (version)
 
@@ -149,23 +156,41 @@ main = do
      exitFailure
   categorizer <- readCategorizer (optCategorizeFile flags)
 
-  captures <- readTimeLog (optLogFile flags)
-  captures `deepseq` return ()
+  timelog <- BS.readFile (optLogFile flags)
+  size <- fileSize <$> getFileStatus (optLogFile flags)
+
+  hSetBuffering stderr NoBuffering
+  trackedTimelog <- trackProgressWithChunkSize (fromIntegral size `div` 100) (\_ b -> do
+    (_height, width) <- getTermSize
+    hPutChar stderr '\r'
+    hPutStr stderr $
+        mkProgressBar (msg "Processing data") percentage (fromIntegral width) (fromIntegral b) (fromIntegral size)
+    when  (fromIntegral b >= fromIntegral size) $ do
+        hPutChar stderr '\r'
+        hPutStr stderr (replicate width ' ')
+        hPutChar stderr '\r'
+    ) timelog
+
+  let captures = parseTimeLog trackedTimelog
   let allTags = categorizer captures
   when (null allTags) $ do
      putStrLn "Nothing recorded yet"
      exitFailure
       
   let filters = (if optAlsoInactive flags then id else (defaultFilter:)) $ optFilters flags
-  let tags = applyFilters filters allTags
   let reps = case optReports flags of {[] -> [TotalTime]; reps -> reverse reps }
 
   -- These are defined here, but of course only evaluated when any report
   -- refers to them. Some are needed by more than one report, which is then
   -- advantageous.
-  let c = prepareCalculations allTags tags
+  let opts = optReportOptions flags
+  let (c,results) = runLeftFold (filterPredicate filters `filterWith` 
+        (pure (,) <*> prepareCalculations <*> processReports opts c reps)) allTags
+
+  -- Force the results a bit, to ensure the progress bar to be shown before the titel
+  c `seq` return ()
   
-  putReports (optReportOptions flags) c reps
+  renderReport opts (MultpleReportResults results)
 
 {-
 import Data.Accessor
